# coding: utf-8

import datetime

from django.db import models
from django.utils import simplejson as json
from django.db.models.signals import pre_delete
from django.dispatch import receiver

from djcelery.models import PeriodicTask, CrontabSchedule
from m3.actions import ApplicationLogicException
from m3.db import BaseEnumerate

from kinder.core.models import BaseReplicatedModel
from kinder.core.unit.models import UnitKind
from kinder.core.unit.models import Unit

from utils import ViewList
from constants import (
    AGE_CATEGORIES_FULL, AGE_CATEGORIES_FULL_CURRENT,
    APPLICATION_STATUSES, AGE_CATEGORIES_FULL_LIST,
    BENEFIT_TYPES, HEALTH_NEEDS, DOO_TYPES, DELIVERY_TYPES,
    GROUP_TYPES, AGE_CATEGORIES_CUT, NO, AGE_CATEGORIES_FULL_MAP,
    AGE_CATEGORIES_CUT_LIST)
import settings as gs_settings


@receiver(pre_delete, sender=Unit)
def delete_unit(instance, **kwargs):

    GisdoUnit.objects.filter(unit=instance).delete()


class InstallationType(BaseEnumerate):

    REGIONAL = 1
    MUNICIPAL = 2

    values = {
        REGIONAL: u'Региональная',
        MUNICIPAL: u'Муниципальная'
    }


class GisdoUnit(BaseReplicatedModel):
    """
    Модель, для расширения учреждений
    """
    unit = models.OneToOneField(Unit, parent_link=True, related_name='gisdo')
    not_on_federal_report = models.BooleanField(default=False)
    doo_identity = models.CharField(max_length=20, verbose_name=u'Идентификатор ДОО для шины', null=True)

    # идентификатор ДОО
    def create_doo_identity(self):
        """
        композитный идентификатор ДОО, заполняется следующим образом:
        xxyyyyyyyyyyyyzzzz,
        xx - код региона (2 знака) из настроек проекта
        yyyyyyyyyyyy - цифры кода ОКТМО (11 знаков)
        zzzz - номер ДОО (поле doo_number)
        """
        octmo_max_length = 11
        doo_number_max_length = 4
        region_max_length = 2

        if self.unit.kind.id != UnitKind.DOU:
            return
        if not gs_settings.REGION_CODE:
            raise ApplicationLogicException(u'Не задан код региона')
        if len(str(gs_settings.REGION_CODE)) > region_max_length:
            raise ApplicationLogicException(
                u'Длина кода региона превышает максимальное значение')

        identity = str(gs_settings.REGION_CODE).rjust(region_max_length, '0')

        octmo = self.unit.get_mo().octmo
        if octmo:
            identity += self.unit.get_mo().octmo.rjust(octmo_max_length, '0')
        else:
            raise ApplicationLogicException(
                u'В МО %s не задан код ОКТМО' % self.unit.get_mo().display())

        identity += str(self.unit.doo_number).rjust(doo_number_max_length, '0')

        self.doo_identity = identity


class ReportForm(BaseReplicatedModel):
    """
    Форма отчетности "Федеральная отчетность"
    """
    date = models.DateTimeField(verbose_name=u'Дата создания', auto_now_add=True)
    sent_time = models.DateTimeField(verbose_name=u'Время отправки', null=True)
    user = models.ForeignKey('users.UserProfile', verbose_name=u'Пользователь', null=True)
    unit = models.ForeignKey('unit.Unit', verbose_name=u'Учреждение', null=True)
    xml = models.TextField(verbose_name=u'Сформированная XML', null=True)
    in_progress = models.BooleanField(default=False, verbose_name=u'Сбор отчета')

    class Meta:
        verbose_name = u"Форма 'Федеральная отчетность'"
        verbose_name_plural = u"Формы 'Федеральная отчетность'"

    def __unicode__(self):
        return u"%s от %s" % (self._meta.verbose_name, self.date)

    @property
    def presentation(self):

        date = self.date.strftime("%d.%m.%Y")

        return u"{date}, {unit}".format(date=date, unit=self.unit)

    @property
    def progress(self):
        return u'В процессе сборки' if self.in_progress else u'Собран'

    @property
    def sent(self):
        return u'Отправлен' if self.sent_time else u'Не отправлен'


class ReportFormRow(BaseReplicatedModel):
    """
    Строка отчета для конкретного учреждения
    """
    report = models.ForeignKey(ReportForm, verbose_name=u'Форма отчетности')
    unit = models.ForeignKey('unit.Unit', verbose_name=u'Учреждение')

    applications = models.TextField(verbose_name=u'Информация по заявлениям', null=True)
    queues = models.TextField(verbose_name=u'Информация по очередям', null=True)
    enrolled = models.TextField(verbose_name=u'Информация о зачисленных детях', null=True)
    capacities = models.TextField(verbose_name=u'Информация о свободных местах', null=True)

    class Meta:
        verbose_name = u"Показатель по учреждению"
        verbose_name_plural = u"Показатели по учреждению"

    def __unicode__(self):
        return u"%s %s на %s" % (self._meta.verbose_name,
                                 self.unit.display(),
                                 self.report.date if self.report else '-')

    def get_applications_view_data(self):
        """
        Данные отчета по учреждению по заявлениям в формате,
        годном для загрузки в грид
        """
        ap = json.loads(self.applications)
        common = u'Общее количество поданных заявлений на постановку в очередь детей %s'
        delivery = u'Общее количество заявлений поданных %s'
        non_state = (u'Общее количество поданных заявлений на постановку в очередь в '
                     u'негосударственные ДОО детей в возрасте %s')
        business = (u'Общее количество поданных заявлений на постановку в очередь у ИП, '
                    u'реализующих программы дошкольного образования, детей в возрасте %s')
        short_stay = (u'Общее количество поданных заявлений на постановку в очередь в '
                      u'группы кратковременного пребывания детей в возрасте %s')

        data = ViewList()

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((common % verbose_age_range, ap['COMMON'][verbose_age_range]))

        if AGE_CATEGORIES_FULL_CURRENT['C2-6-MONTHS'] in ap['COMMON']:

            for i in range(15):
                verbose_age_range = AGE_CATEGORIES_FULL_CURRENT['C' + AGE_CATEGORIES_FULL_LIST[i]]
                data.append((common % verbose_age_range, ap['COMMON'][verbose_age_range]))

        data.append((delivery % DELIVERY_TYPES['PORTAL'], ap['DELIVERY'][DELIVERY_TYPES['PORTAL']]))
        data.append((delivery % DELIVERY_TYPES['IN_PERSON'], ap['DELIVERY'][DELIVERY_TYPES['IN_PERSON']]))

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((non_state % verbose_age_range, ap['DOO_TYPE'][DOO_TYPES['NON_STATE']][verbose_age_range]))

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((business % verbose_age_range, ap['DOO_TYPE'][DOO_TYPES['BUSINESS']][verbose_age_range]))

        if AGE_CATEGORIES_FULL_CURRENT['C2-6-MONTHS'] in ap['DOO_TYPE'][DOO_TYPES['NON_STATE']]:
            for i in range(15):
                verbose_age_range = AGE_CATEGORIES_FULL_CURRENT['C' + AGE_CATEGORIES_FULL_LIST[i]]
                data.append((non_state % verbose_age_range, ap['COMMON'][verbose_age_range]))

        if AGE_CATEGORIES_FULL_CURRENT['C2-6-MONTHS'] in ap['DOO_TYPE'][DOO_TYPES['BUSINESS']]:
            for i in range(15):
                verbose_age_range = AGE_CATEGORIES_FULL_CURRENT['C' + AGE_CATEGORIES_FULL_LIST[i]]
                data.append((business % verbose_age_range, ap['COMMON'][verbose_age_range]))

        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((short_stay % verbose_age_range, ap['GROUP_TYPE'][GROUP_TYPES['SHORT_STAY']]
                                                                         [verbose_age_range]))

        if AGE_CATEGORIES_FULL_CURRENT['C2-6-MONTHS'] in ap['GROUP_TYPE'][GROUP_TYPES['SHORT_STAY']]:
            for i in range(15):
                verbose_age_range = AGE_CATEGORIES_FULL_CURRENT['C' + AGE_CATEGORIES_FULL_LIST[i]]
                data.append((short_stay % verbose_age_range, ap['COMMON'][verbose_age_range]))

        data.append((u'Количество отказов', ap['STATUS'][APPLICATION_STATUSES['DENIED']]))

        return data.get_data()

    def get_queues_view_data(self):
        """
        Данные отчета по учреждению по очередям в формате,
        годном для загрузки в грид
        """
        qu = json.loads(self.queues)
        common = u'Количество детей в очереди (отложенный спрос) в возрасте %s'
        want_change_doo = u'Количество детей в очереди (с учетом детей, желающих сменить ДОО) в возрасте %s'
        benefits = u'Количество детей в очереди, имеющих право на льготное зачисление в возрасте %s'
        benefit_type = u'Количество детей в очереди, имеющих право на льготное зачисление на основании %s льгот'
        disabled = u'Количество детей в очереди с ограниченными возможностями здоровья в возрасте %s'
        needs_by_group = u'Количество детей в очереди, нуждающихся в %s группах'
        non_state = u'Количество детей в очереди в негосударственные ДОО в возрасте %s'
        business = u'Количество детей в очереди у ИП, реализующих программы дошкольного образования, в возрасте %s'
        short_stay = u'Количество детей в очереди в группы кратковременного пребывания в возрасте %s'
        actual = u'Количество детей в очереди, необеспеченных местом в ДОО в возрасте %s (актуальный спрос)'
        unsecured = (u'Количество детей в очереди необеспеченных местом в ДОО в возрасте %s '
                     u'(актуальный спрос на текущую дату)')
        registered = u'Количество детей в очереди (отложенный спрос) в возрасте %s (на текущую дату)'
        common_queue = (u'Численность детей, поставленных на учет для предоставления места в дошкольных'
                        u' образовательных организациях в возрасте %s')

        data = ViewList()

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((common % verbose_age_range, qu['STATUS'][APPLICATION_STATUSES['IN_LINE_DEFERRED']]
                                                                 [verbose_age_range]))
        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((want_change_doo % verbose_age_range, qu['STATUS']
                                                                [APPLICATION_STATUSES['IN_LINE_WANT_CHANGE_DOO']]
                                                                [verbose_age_range]))
        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((benefits % verbose_age_range, qu['BENEFITS'][BENEFIT_TYPES['HAVE_BENEFITS']]
                                                                     [verbose_age_range]))
        data.append((benefit_type % u'федеральных', qu['BENEFITS'][BENEFIT_TYPES['FEDERAL']]))
        data.append((benefit_type % u'региональных', qu['BENEFITS'][BENEFIT_TYPES['REGIONAL']]))

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((disabled % verbose_age_range, qu['HEALTH_NEEDS'][HEALTH_NEEDS['DISABLED']][verbose_age_range]))

        data.append((needs_by_group % u'компенсирующих', qu['HEALTH_NEEDS'][HEALTH_NEEDS['CORRECT']]))
        data.append((needs_by_group % u'оздоровительных', qu['HEALTH_NEEDS'][HEALTH_NEEDS['HEALTH']]))

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((non_state % verbose_age_range, qu['DOO_TYPE'][DOO_TYPES['NON_STATE']][verbose_age_range]))

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((business % verbose_age_range, qu['DOO_TYPE'][DOO_TYPES['BUSINESS']][verbose_age_range]))

        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((short_stay % verbose_age_range, qu['GROUP_TYPE'][GROUP_TYPES['SHORT_STAY']]
                                                                         [verbose_age_range]))
        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((actual % verbose_age_range, qu['DOO_TYPE'][NO][verbose_age_range]))
        # В старых отчетах этот показатель не вычислялся
        if (AGE_CATEGORIES_FULL_CURRENT['C2-6-MONTHS'] in qu['DOO_TYPE'][NO]
                and AGE_CATEGORIES_FULL_CURRENT['C2-6-MONTHS'] in
                qu['STATUS'][APPLICATION_STATUSES['IN_LINE_DEFERRED']]):

            for i in range(15):
                verbose_age_range = AGE_CATEGORIES_FULL_CURRENT['C' + AGE_CATEGORIES_FULL_LIST[i]]
                data.append((unsecured % AGE_CATEGORIES_FULL_MAP[verbose_age_range], qu['DOO_TYPE'][NO]
                                                                                       [verbose_age_range]))
            for i in range(15):
                verbose_age_range = AGE_CATEGORIES_FULL_CURRENT['C' + AGE_CATEGORIES_FULL_LIST[i]]
                data.append((registered % AGE_CATEGORIES_FULL_MAP[verbose_age_range], qu['STATUS']
                                                                 [APPLICATION_STATUSES['IN_LINE_DEFERRED']]
                                                                 [verbose_age_range]))
        if AGE_CATEGORIES_FULL_CURRENT['C2-6-MONTHS'] in qu['COMMON']:
            for i in range(15):
                verbose_age_range = AGE_CATEGORIES_FULL_CURRENT['C' + AGE_CATEGORIES_FULL_LIST[i]]
                data.append((common_queue % AGE_CATEGORIES_FULL_MAP[verbose_age_range], qu['DOO_TYPE'][NO]
                                                                                       [verbose_age_range]))

        return data.get_data()

    def get_enrolled_view_data(self):
        """
        Данные отчета по учреждению по зачисленным детям в формате,
        годном для загрузки в грид
        """
        en = json.loads(self.enrolled)
        common = u'Количество зачисленных детей в возрасте %s'
        disabled = (u'Количество детей, зачисленных в группы для детей с '
                    u'ограниченными возможностями здоровья в возрасте %s')
        non_state = u'Количество детей, зачисленных  в негосударственные ДОО, в возрасте %s'
        business = u'Количество детей, зачисленных у ИП, реализующих программы дошкольного образования, в возрасте %s'
        short_stay = u'Количество детей, зачисленных в группы кратковременного пребывания, в возрасте %s'
        care_non_state = (u'Количество детей, зачисленных в группы по присмотру и уходу в '
                          u'негосударственных ДОО, для детей в возрасте %s')
        care_business = (u'Количество детей, зачисленных в группы по присмотру и уходу в ИП, реализующих '
                         u'программы дошкольного образования,  в возрасте  %s')
        inline_enrolled = u'Количество детей в возрасте %s'
        all_childs = u'Общее количество детей в возрасте %s'

        count_people_current_date = u'Численность детей, посещающих дошкольную образовательную организацию (%s)'

        data = ViewList()

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((common % verbose_age_range, en['COMMON'][verbose_age_range]))

        data.append((u'Количество детей, желающих сменить ДОО', en['STATUS'][APPLICATION_STATUSES['WANT_CHANGE_DOO']]))
        data.append((u'Количество детей, зачисленных в семейные дошкольные группы',
                     en['GROUP_TYPE'][GROUP_TYPES['FAMILY']]))

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((disabled % verbose_age_range, en['GROUP_TYPE'][GROUP_TYPES['DISABLED']][verbose_age_range]))

        data.append((u'Количество детей, зачисленных в компенсирующих группах',
                     en['GROUP_TYPE'][GROUP_TYPES['CORRECT']]))
        data.append((u'Количество детей, зачисленных в оздоровительных группах',
                     en['GROUP_TYPE'][GROUP_TYPES['HEALTH']]))

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((non_state % verbose_age_range, en['DOO_TYPE'][DOO_TYPES['NON_STATE']][verbose_age_range]))

        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((business % verbose_age_range, en['DOO_TYPE'][DOO_TYPES['BUSINESS']][verbose_age_range]))

        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((short_stay % verbose_age_range,
                         en['GROUP_TYPE'][GROUP_TYPES['SHORT_STAY']][verbose_age_range]))
        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((care_non_state % verbose_age_range,
                         en['GROUP_TYPE'][GROUP_TYPES['CARE']][DOO_TYPES['NON_STATE']][verbose_age_range]))
        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((care_business % verbose_age_range,
                        en['GROUP_TYPE'][GROUP_TYPES['CARE']][DOO_TYPES['BUSINESS']][verbose_age_range]))
        for i in range(15):
            verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
            data.append((inline_enrolled % verbose_age_range,
                         en['STATUS'][APPLICATION_STATUSES['IN_LINE_ENROLLED']][verbose_age_range]))

        # В старых отчетах этот показатель не вычислялся
        if 'ALL_CHILDS' in en:

            for i in range(15):
                verbose_age_range = AGE_CATEGORIES_FULL[AGE_CATEGORIES_FULL_LIST[i]]
                data.append((all_childs % verbose_age_range, en['ALL_CHILDS'][verbose_age_range]))

        if AGE_CATEGORIES_FULL_CURRENT['C2-6-MONTHS'] in en['COMMON']:

            for i in range(15):
                verbose_age_range = AGE_CATEGORIES_FULL_CURRENT['C' + AGE_CATEGORIES_FULL_LIST[i]]
                data.append((count_people_current_date % AGE_CATEGORIES_FULL_MAP[verbose_age_range],
                             en['COMMON'][verbose_age_range]))
        return data.get_data()

    def get_capacities_view_data(self):
        """
        Данные отчета по учреждению по свободным местам в формате,
        годном для загрузки в грид
        """
        ca = json.loads(self.capacities)
        common = u'Количество свободных мест на дату сбора отчета в группы в возрасте %s'
        disabled = (u'Количество свободных мест в группах для детей с ограниченными '
                    u'возможностями здоровья в возрасте %s')
        non_state = u'Количество свободных мест в негосударственных ДОО в группы в возрасте %s'
        business = (u'Количество свободных мест у ИП, реализующих программы '
                    u'дошкольного образования, в группы в возрасте %s')
        short_stay = u'Количество свободных мест в группах кратковременного пребывания в возрасте %s'

        data = ViewList()

        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((common % verbose_age_range, ca['COMMON'][verbose_age_range]))

        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((disabled % verbose_age_range, ca['GROUP_TYPE'][GROUP_TYPES['DISABLED']][verbose_age_range]))

        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((non_state % verbose_age_range, ca['DOO_TYPE'][DOO_TYPES['NON_STATE']][verbose_age_range]))

        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((business % verbose_age_range, ca['DOO_TYPE'][DOO_TYPES['BUSINESS']][verbose_age_range]))

        for i in range(3):
            verbose_age_range = AGE_CATEGORIES_CUT[AGE_CATEGORIES_CUT_LIST[i]]
            data.append((short_stay % verbose_age_range,
                         ca['GROUP_TYPE'][GROUP_TYPES['SHORT_STAY']][verbose_age_range]))
        return data.get_data()


class ScheduleSettings(models.Model):
    """
    Модель настроек celery beat
    """
    PK = 1
    push_login = models.CharField(
        u'Логин для отправки', max_length=30, null=True)
    push_password = models.CharField(
        u'Пароль для отправки', max_length=30, null=True)

    time = models.TimeField(verbose_name=u'Время начала', null=True)
    is_active = models.BooleanField(
        verbose_name=u'Включение автоматического сбора',
        default=False)
    resend_count = models.PositiveSmallIntegerField(
        verbose_name=u'Количество повторных отправок при неудаче',
        default=3)
    zip_xml = models.BooleanField(
        verbose_name=u'Кодировать данные при отправке в ZIP',
        default=False)
    async_send = models.BooleanField(
        verbose_name=u'Асинхронная отправка отчета', default=False)
    system_name = models.CharField(
        u'Наименование системы', max_length=250, null=True)
    email = models.EmailField(
        u'E-mail контактного лица', max_length=30, null=True)
    installation_type = models.SmallIntegerField(
        choices=InstallationType.get_choices(), null=True)

    def save(self, force_insert=False, force_update=False, using=None):
        self.pk = self.PK
        super(ScheduleSettings, self).save(force_update, force_update, using)

    @staticmethod
    def get_settings():
        ss, created = ScheduleSettings.objects.get_or_create(id=ScheduleSettings.PK,
                                                             defaults={'id': ScheduleSettings.PK})
        return ss


class TaskScheduler(models.Model):
    """
    Планировщик для сбора данных
    """
    periodic_task = models.ForeignKey(PeriodicTask)

    @staticmethod
    def schedule_every(task_name, hour, minute, args='[]', kwargs='{}'):
        """
        schedules a task by name
        """
        ptask_name = "%s_%s" % (task_name, datetime.datetime.now())
        crontab_schedules = CrontabSchedule.objects.filter(hour=hour, minute=minute,
                                                           day_of_week='*', day_of_month='*', month_of_year='*')
        crontab_schedule = (crontab_schedules[0]
                            if crontab_schedules.exists()
                            else CrontabSchedule.objects.create(hour=hour, minute=minute))
        ptask = PeriodicTask.objects.create(name=ptask_name,
                                            task=task_name,
                                            crontab=crontab_schedule,
                                            args=args,
                                            kwargs=kwargs)
        return TaskScheduler.objects.create(periodic_task=ptask)

    def stop(self):
        self.periodic_task.enabled = False
        self.periodic_task.save()

    def start(self):
        self.periodic_task.enabled = True
        self.periodic_task.save()

    def terminate(self):
        self.stop()
        self.delete()
        self.periodic_task.delete()